package de.hawhamburg.reachability.sensor.adapter;

import org.apache.log4j.Logger;

import com.google.gson.Gson;

import de.hawhamburg.reachability.reasoner.enumeration.ReasonerDimension;
import de.hawhamburg.reachability.sensor.Sensor;
import de.hawhamburg.reachability.sensor.SensorAdapter;
import de.hawhamburg.reachability.sensor.SensorAdapterFactory;
import de.hawhamburg.reachability.sensor.SensorAgent;
import de.hawhamburg.reachability.sensor.SensorData;
import de.hawhamburg.reachability.sensor.adapter.json.JsonUbiSenseMessage;
import de.hawhamburg.reachability.util.ConnectionUtil;

/**
 * Sensor to locate the inhabitants position.<br><br>
 * 
 * This sensor adapts the LP3DObjectLocation agent (by Bastian Karstead). It evaluates the actual position of the
 * inhabitant and provides it as {@link SensorData}. The sensor data will be received via the ActiveMQ topic. It isn't 
 * necessary to send requests to the sensor agent in this version of sensor agent.  
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public class LP3DObjectLocationAdapter extends    Sensor
                                       implements SensorAdapter  {
    
    /**
     * Creates a new LP3DObjectLocation adapter which provides the current position of the 
     * inhabitant.<br><br>
     * 
     * The constructor of {@link Sensor} loads the local XML configuration from 
     * {@link de.hawhamburg.reachability.sensor.Settings#SENSOR_CONFIGURATION_PATH}, which is named like this class, 
     * and opens all required ActiveMQ connections.
     * 
     * @param dimension The {@link ReasonerDimension} for which this sensor supplies data.
     */
    public LP3DObjectLocationAdapter(final ReasonerDimension dimension) {
        super(dimension);
    }

    /**
     * Provides the inhabitants current position inside the environment as {@link SensorData}.
     * 
     * @return          The inhabitants current position as {@link SensorData}.
     */
    public final SensorData getSensorOutput() {
        String json = ConnectionUtil.getMessageOnTopic(this.getResultTopic());
        
        if (json != null) {
            Gson g = new Gson();
            JsonUbiSenseMessage msgObj = g.fromJson(json, JsonUbiSenseMessage.class);
            
            double[] point = {msgObj.NewPosition.X, msgObj.NewPosition.Y};
            return new SensorData(this.getClass().getSimpleName(), this.getDataDimension(), point);
        } else {
            // No data found...
            return null;
        }
    }

    /**
     * Main method which starts a single sensor adapter for each context dimension to sense simulated sensor data.
     * 
     * @param args      The start arguments as array of {@link String}.
     */
    public static void main(final String[] args) {
        // Remove all existing loggers for a clear console
        Logger.getRootLogger().removeAllAppenders();
        
        // Build this sensor adapter by using the SensorAdapterFactory
        SensorAdapter sa = SensorAdapterFactory.getConnectedSensorAdapter(
                                LP3DObjectLocationAdapter.class.getSimpleName(),
                                ReasonerDimension.position);
        
        // Start the sensor adapter as sensor agent, which supplied the required control functions
        new SensorAgent(sa);
    }
}
